#!/usr/bin/env python
#Author: Mark Feineigle
#Create Date: 2018/03/16
#Last Modify: 2018/03/16
# Building Machine Learning Systems with Python
# page 20

import scipy as sp
# Load the tab delimited data
data = sp.genfromtxt("web_traffic.tsv", delimiter="\t")
print data[:10]
print data.shape

# The first vector, x, will contain the hours and the other, y, will contain
# the web hits in that particular hour.
x = data[:,0]
y = data[:,1]

# Let us check how many hours contain invalid data.
print sp.sum(sp.isnan(y))

# Since only 8 are missing out of 743, we can delete them.
x = x[~sp.isnan(y)]
y = y[~sp.isnan(y)]

# Plot to get a feel for the data.
import matplotlib.pyplot as plt
plt.scatter(x,y)
plt.title("Web traffic over the last month")
plt.xlabel("Time")
plt.ylabel("Hits/hour")
plt.xticks([w*7*24 for w in range(10)], ['week %i'%w for w in range(10)])
plt.autoscale(tight=True)
plt.grid()
#plt.show()

# Calculate the squared distance of the prediction to the real.
def error(f, x, y):
  return sp.sum((f(x)-y)**2)

# 1st degree polynomial model. full=True generates the residuals, ranks, sv,
# and rcond results. fp1 is the model parameters.
fp1, residuals, rank, sv, rcond = sp.polyfit(x, y, 1, full=True)

print "Model parameters: %s" % fp1
#Model parameters: [2.59619213  989.02487106]
print "Res: %s" % res
#[ 3.17389767e+08]
# This means that the best straight line fit is the following function:
# f(x) = 2.59619213 * x + 989.02487106.

# We then use poly1d() to create a model function from the model parameters.
f1 = sp.poly1d(fp1)
print(error(f1, x, y))
#317389767.34

# We can now use f1() to plot our first trained model. 
fx = sp.linspace(0,x[-1], 1000) # generate X-values for plotting
plt.plot(fx, f1(fx), linewidth=4)
plt.legend(["d=%i" % f1.order], loc="upper left")
#plt.show()

# Try a 2nd degree polynomial model.
f2p = sp.polyfit(x, y, 2)
print(f2p)
#array([ 1.05322215e-02, -5.26545650e+00,
f2 = sp.poly1d(f2p)
print(error(f2, x, y))
#179983507.878
plt.plot(fx, f2(fx), linewidth=4)
plt.legend(["d=%i" % f2.order], loc="upper left")
plt.show()

# It seems that there is an inflection point between weeks 3 and 4. So let us
# separate the data and train two lines using week 3.5 as a separation point.
inflection = 3.5*7*24 # calculate the inflection point in hours

xa = x[:inflection] # data before the inflection point
ya = y[:inflection]
xb = x[inflection:] # data after
yb = y[inflection:]

fa = sp.poly1d(sp.polyfit(xa, ya, 1))
fb = sp.poly1d(sp.polyfit(xb, yb, 1))

fa_error = error(fa, xa, ya)
fb_error = error(fb, xb, yb)
print("Error inflection=%f" % (fa + fb_error))
#Error inflection=156,639,407.701523

#TODO
# plot these
# split the data into test and train sets

# Answer the initial question of when 100k hits per hour are expected.
from scipy.optimize import fsolve
reached_max = fsolve(fbt2-100000, 800)/(7*24)
print("100,000 hits/hour expected at week %f" % reached_max[0])
#100,000 hits/hour expected at week 9.827613

